/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggraxl.Auftrag
 *
 * de.inovat.dua.aggraxl.Auftrag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggraxl.Auftrag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggraxl.Auftrag; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggraxl;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;
import java.util.Date;

//~ KLASSEN ===================================================================

/**
 * Enthält alle Informationen für einen Aggregationsauftrag.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Auftrag {

    /** IntervallBeginn */
    private long _intervallBeginn;

    /** IntervallEnde: Intervallbeginn + Intervalllänge (ausschließlich!, also der nächste Intervallbeginn */
    private long _intervallEnde;

    /** Legt fest, ob Auftrag genau einmal OHNE weitere Folgeaufträge abgewickelt werden soll. */
    private boolean _isEinmalAuftrag;

    /** VewAXL - Verwaltungsobjekt, welches den Auftrag ausführen soll */
    private VewAXL _vewAXL;

    /** Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970 */
    private long _ausfuehrungsZeitPunkt = 0;

    /** Datumsformatierung für Ausgabe toString() */
    SimpleDateFormat _datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor, um einen Folgeauftrag auf Basis eines bestehenden Auftrags zu berechnen
     *
     * @param auftrag Aktueller Auftrag, für den der Folgeauftrag bestimmt werden soll
     */
    public Auftrag(Auftrag auftrag) {
        long intervallDauer    = auftrag.getIntervallEnde() - auftrag.getIntervallBeginn();
        long minimaleWarteZeit = auftrag.getAusfuehrungsZeitPunkt() - auftrag.getIntervallEnde();

        this._intervallBeginn       = auftrag.getIntervallBeginn() + intervallDauer;
        this._intervallEnde         = auftrag.getIntervallEnde() + intervallDauer;
        this._vewAXL                = auftrag.getVewAXL();
        this._ausfuehrungsZeitPunkt = this._intervallEnde + minimaleWarteZeit;
        this._isEinmalAuftrag       = auftrag.isEinmalAuftrag();
    }

    /**
     * Konstruktor, um den initialen Erstauftrag für ein Verwaltungsobjekt und das erste zu berechnende Intervall
     * anzulegen
     *
     * @param ausfuehrungsZeitpunkt Zeitpunkt, zu dem Auftrag frühestens ausgeführt werden soll.
     * @param intervallBeginn       Intervallbeginn (EINSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param intervallEnde         Intervallende (AZSSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param vewAXL                Verwaltungsobjekt, welches den Auftrag letztendlich ausführen soll
     * @param isEinmalAuftrag       <code>true</code>, wenn es sich um einen einmalig auszuführenden Auftrag OHNE
     *                              Folgeauftrag handelt, sonst <code>false</code>.
     */
    public Auftrag(long ausfuehrungsZeitpunkt, long intervallBeginn, long intervallEnde, VewAXL vewAXL, boolean isEinmalAuftrag) {
        _ausfuehrungsZeitPunkt = ausfuehrungsZeitpunkt;
        _intervallBeginn       = intervallBeginn;
        _intervallEnde         = intervallEnde;
        _vewAXL                = vewAXL;
        _isEinmalAuftrag       = isEinmalAuftrag;
    }

    //~ METHODEN ==============================================================

    /**
     * Führt den Auftrag aus, indem am zugeordneten Verwaltungsobjekt die Methode {@link
     * de.inovat.dua.aggraxl.VewAXL#ausfuehren(Auftrag)} aufgerufen wir.
     *
     * @return Meldung mit dem Ergebnis der Ausführung des Auftrags.
     */
    public String ausfuehren() {
        return _vewAXL.ausfuehren(this);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert den Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     */
    public long getAusfuehrungsZeitPunkt() {
        return _ausfuehrungsZeitPunkt;
    }

    /**
     * Liefert den Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallBeginn() {
        return _intervallBeginn;
    }

    /**
     * Liefert den Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallEnde() {
        return _intervallEnde;
    }

    /**
     * Liefert das Verwaltungsobjekt, welches den Auftrag ausführen soll.
     *
     * @return Verwaltungsobjekt, welches den Auftrag ausführen soll.
     */
    public VewAXL getVewAXL() {
        return _vewAXL;
    }

    /**
     * Test, ob es sich um einen Einmalauftrag oder um einen Auftrag mit zyklischer Wiederholung handelt.
     *
     * @return <code>true</code>, wenn es sich um einen einmalig auszuführenden Auftrag OHNE Folgeauftrag handelt, sonst
     *         <code>false</code>.
     */
    public boolean isEinmalAuftrag() {
        return _isEinmalAuftrag;
    }

    //~ METHODEN ==============================================================

    @Override

    /**
     * Gibt die Textrepräsentation für einen Auftrag aus.
     */
    public String toString() {
        String ausfuehrung = "DauerAuftrag ";

        if (_isEinmalAuftrag) {
            ausfuehrung = "EinmalAuftrag";
        }

        return String.format("Auftrag (%s) %s: %s - %s fuer %s", ausfuehrung, _datumsFormatierer.format(new Date(_ausfuehrungsZeitPunkt)), _datumsFormatierer.format(new Date(_intervallBeginn)), _datumsFormatierer.format(new Date(_intervallEnde)), _vewAXL.getPid());
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.12.10
